package cn.efunbox.audio.controller;

import cn.efunbox.audio.consts.Status;
import cn.efunbox.audio.entity.Channel;
import cn.efunbox.audio.entity.Grouping;
import cn.efunbox.audio.service.AdminService;
import cn.efunbox.audio.service.GroupingService;
import cn.efunbox.audio.service.TrailService;
import cn.efunbox.audio.utils.ApiCode;
import cn.efunbox.audio.utils.Common;
import cn.efunbox.audio.utils.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.sql.Timestamp;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by yao on 17-9-26.
 */
@RestController
@Slf4j
@RequestMapping(value = "/grouping")
public class GroupingController {

    @Autowired
    GroupingService groupingService;
    @Autowired
    AdminService adminService;
    @Autowired
    TrailService trailService;

    @RequestMapping(value = "/search" ,method = RequestMethod.POST)
    public void Search(HttpServletRequest request, HttpServletResponse response){
        String name = request.getParameter("name");
        String idGroup = request.getParameter("idGroup");
        String idFather = request.getParameter("idFather");
        String page = request.getParameter("page");
        String size = request.getParameter("size");
        if(page==null || page.length()<1)
            page = "0";
        if(size==null || size.length()<1)
            size = "0";
//        if(name==null && idGroup==null && idFather==null){
//            HttpUtil.responseApiCode(request, response, ApiCode.PARAMETER_ERROR);
//            return;
//        }
        Map<String,Object> map = new HashMap<>();
        List<Grouping> list = null;
        if(idGroup!=null && idGroup.length()>0)
            list = groupingService.SearchById(Long.valueOf(idGroup));
        else if(name!=null && name.length()>0)
            list = groupingService.SearchByName(name);
        else if(idFather!=null && idFather.length()>0)
            list = groupingService.SearchByIdFather(Long.valueOf(idFather));
        else{
            Page<Grouping> pageRes = groupingService.SearchAll(Integer.valueOf(page), Integer.valueOf(size));
            map.put("data", pageRes.getContent());
            map.put("size", pageRes.getNumberOfElements());
            map.put("total", pageRes.getTotalElements());
        }

        if(list!=null){
            map.put("data", list);
            map.put("size", list.size());
            map.put("total", list.size());
        }

        HttpUtil.responseOkData(request, response, map);
        return;
    }

//    @RequestMapping(value = "/insert", method = RequestMethod.POST)
//    public void Insert(HttpServletRequest request, HttpServletResponse response){
//        String name = request.getParameter("name");
//        String idFather = request.getParameter("idFather");
//        if(name==null){
//            HttpUtil.responseApiCode(request, response, ApiCode.PARAMETER_ERROR);
//            return;
//        }
//        if(idFather==null || idFather.length()==0)
//            idFather = "0";
//
//        List<Grouping> list = groupingService.SearchByName(name);
//        if(list!=null && list.size()>0){
//            HttpUtil.responseApiCode(request, response, ApiCode.RECORD_EXIST);
//            return;
//        }
//        Grouping grouping = new Grouping();
//        grouping.setName(name);
//        grouping.setIdFather(Long.valueOf(idFather));
//        grouping = groupingService.Insert(grouping);
//        HttpUtil.responseOutWithJson(request, response, grouping);
//        return;
//    }

    @RequestMapping(value = "/insert", method = RequestMethod.POST)
    public void Insert(HttpServletRequest request, HttpServletResponse response, @RequestBody Grouping grouping){
        if(grouping.getName()==null){
            HttpUtil.responseApiCode(request, response, ApiCode.PARAMETER_ERROR);
            return;
        }
        List<Grouping> list = groupingService.SearchByName(grouping.getName());
        if(list!=null && list.size()>0){
            HttpUtil.responseApiCode(request, response, ApiCode.RECORD_EXIST);
            return;
        }
        grouping.setHasSon(false);
        grouping.setStatus(Status.ONLINE.getCode());
        grouping.setCreated(new Timestamp(new Date().getTime()));
        grouping = groupingService.Insert(grouping);
        //增加父节点的子节点标记
        if(grouping.getIdFather()>0){
            Grouping g = groupingService.GetOne(grouping.getIdFather());
            g.setHasSon(true);
            groupingService.Insert(g);
        }

        TrailController.Insert(adminService, trailService, request, "grouping", grouping.getId(), " 新建资源分组：" + grouping.toString());

        HttpUtil.responseOkData(request, response, grouping);
        return;
    }

    @RequestMapping(value = "/update", method = RequestMethod.POST)
    public void Update(HttpServletRequest request, HttpServletResponse response, @RequestBody Grouping grouping){
        Grouping groupingOld = groupingService.GetOne(grouping.getId());
        if(groupingOld==null){
            HttpUtil.responseApiCode(request, response, ApiCode.PARAMETER_ERROR);
            return;
        }
        grouping = (Grouping) Common.CopyValue(grouping, groupingOld);
        if(groupingOld.getHasSon())
            grouping.setHasSon(true);
        grouping = groupingService.Insert(grouping);

        TrailController.Insert(adminService, trailService, request, "grouping", grouping.getId(), " 更新资源分组：" + grouping.toString());

        HttpUtil.responseOkData(request, response, grouping);
        return;
    }

    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    public void Delete(HttpServletRequest request, HttpServletResponse response){
        String idGroup = request.getParameter("idGroup");
        Grouping grouping = groupingService.GetOne(Long.valueOf(idGroup));
        groupingService.Delete(Long.valueOf(idGroup));
        //判断父节点是否还有子节点
        if(grouping.getIdFather()>0){
            List<Grouping> list = groupingService.SearchByIdFather(grouping.getIdFather());
            if(list==null || list.size()==0){
                Grouping g = groupingService.GetOne(grouping.getIdFather());
                if(g!=null){
                    g.setHasSon(false);
                    groupingService.Insert(g);
                }
            }
        }

        TrailController.Insert(adminService, trailService, request, "grouping", grouping.getId(), " 删除资源分组：" + grouping.toString());

        HttpUtil.responseOk(request, response);
        return;
    }

}
